<?php
/**
 * @package modules
 * @subpackage roles module
 * @category Xaraya Web Applications Framework
 * @version 2.4.0
 * @copyright see the html/credits.html file in this release
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://www.xaraya.com
 * @link http://xaraya.com/index.php/release/27.html
 */
/* Include the base class */
sys::import('modules.dynamicdata.class.properties');
/**
 * Handle Social Profiles property
 * @author mikespub <mikespub@xaraya.com>
 */
class SocialProfilesProperty extends DataProperty
{
    public $id         = 2828;
    public $name       = 'socialprofiles';
    public $desc       = 'Social Profiles';
    public $reqmodules = array('roles');

    function __construct(ObjectDescriptor $descriptor)
    {
        parent::__construct($descriptor);
        $this->tplmodule = 'roles';
        $this->template = 'socialprofiles';
        $this->filepath = 'modules/roles/xarproperties';
    }

    public function checkInput($name = '', $value = null)
    {
        $name = empty($name) ? 'dd_'.$this->id : $name;
        // store the fieldname for validations who need them (e.g. file uploads)
        $this->fieldname = $name;

        if (!isset($value)) {
            if (!xarVarFetch($name, 'array', $value, array(), XARVAR_NOT_REQUIRED)) return;

            $cleanval = array();
            // check for any user variable(s) given for each profile
            foreach (array_keys($value) as $site) {
                foreach ($value[$site] as $key => $val) {
                    // skip empty variables
                    if (empty($val)) continue;
                    // prepare variables before storing them
                    $cleanval[$site][$key] = str_replace('@', '&#064;', xarVarPrepForDisplay($val));
                }
            }
            $value = $cleanval;
        }
        return $this->validateValue($value);
    }

    public function validateValue($value = null)
    {
        if (!parent::validateValue($value)) return false;

        if (!is_array($value)) {
            $this->value = null;
            return false;
        }

// CHECKME: for efficiency reasons, we could preset all profile information in validateValue(),
//          but then it wouldn't be automatically updated if we change the profiles configuration
/*
        $value = $this->getUserProfiles($value);
*/

        $this->setValue($value);
        return true;
    }

    function setValue($value=null)
    {
        if (!empty($value) && !is_array($value)) {
            $this->value = $value;
        } else {
            if (empty($value)) $value = array();
            $this->value = serialize($value);
        }
    }

    public function getValue()
    {
        try {
            $value = unserialize($this->value);
        } catch(Exception $e) {
            $value = array();
        }
        return $value;
    }

    public function showInput(Array $data = array())
    {
        if (!isset($data['value'])) $data['value'] = $this->getValue();
        if (!empty($data['value']) && !is_array($data['value'])) {
            try {
                $data['value'] = unserialize($data['value']);
            } catch(Exception $e) {
                $data['value'] = array();
            }
        }

// TODO: sort the profiles by order of sites in value ?
        $data['profiles'] = $this->getAllProfiles();

        return parent::showInput($data);
    }

    public function showOutput(Array $data = array())
    {
        if (!isset($data['value'])) $data['value'] = $this->getValue();
        if (!empty($data['value']) && !is_array($data['value'])) {
            try {
                $data['value'] = unserialize($data['value']);
            } catch(Exception $e) {
                $data['value'] = array();
            }
        }

// CHECKME: for efficiency reasons, we could preset all profile information in validateValue(),
//          but then it wouldn't be automatically updated if we change the profiles configuration

        $data['value'] = $this->getUserProfiles($data['value']);

        return parent::showOutput($data);
    }

    /**
     * Get all defined profiles
     */
    public function getAllProfiles()
    {
        if (!empty($this->configuration)) {
            if (is_array($this->configuration)) {
                return $this->configuration;
            }
            try {
                $profiles = unserialize($this->configuration);
                return $profiles;
            } catch (Exception $e) {
            }
        }
        return $this->defaultProfiles();
    }

    /**
     * Fill in the profiles with user information
     */
    public function getUserProfiles($userinfo = array())
    {
        // get the currently defined profiles
        $profiles = $this->getAllProfiles();
        foreach ($profiles as $profile) {
            // skip disabled profiles
            if (empty($profile[5])) continue;
            $site = $profile[0];
            // check if we have any user variables for this profile
            if (empty($userinfo[$site])) continue;

            $link = $profile[2];
            $search = array();
            $replace = array();
            // replace the variables in the profile
            foreach ($profile[3] as $var) {
                if (!empty($userinfo[$site][$var])) {
                    $search[] = '{' . $var . '}';
                    $replace[] = $userinfo[$site][$var];
                }
            }
            $link = str_replace($search,$replace,$link);
            // set the profile information for output (use _info_ to avoid conflict with variables)
            $userinfo[$site]['_info_'] =  array('site'    => $profile[0],
                                                'url'     => $profile[1],
                                                'profile' => $profile[2],
                                                'link'    => $link,
                                                'icon'    => $profile[4]);
        }
        return $userinfo;
    }

/*
    public function parseConfiguration($configuration = '')
    {
        // nothing to see here
        return parent::parseConfiguration($configuration);
    }
*/

    public function showConfiguration(Array $data = array())
    {
        // get the profiles from configuration if necessary
        if (!isset($data['configuration'])) $data['configuration'] = $this->configuration;

        // use the default profiles if none were set
        if (empty($data['configuration'])) {
            $data['configuration'] = $this->getAllProfiles();

        // switch to array format if necessary
        } elseif (!is_array($data['configuration'])) {
            try {
                $newconfig = unserialize($data['configuration']);
                $data['configuration'] = $newconfig;
                unset($newconfig);
            } catch (Exception $e) {
                $data['configuration'] = $this->getAllProfiles();
            }
        }

        // add a new blank profile as example
        $data['configuration'][] = array('Add New',
                                         '',
                                         '',
                                         array(),
                                         'contact/new.png',
                                         0,
                                         999);

        return parent::showConfiguration($data);
    }

    public function updateConfiguration(Array $data = array())
    {
        extract($data);
        $valid = false;
        // in case we need to process additional input fields based on the name
        $name = empty($name) ? 'dd_'.$this->id : $name;

        // do something with the configuration and save it in $this->configuration
        if (isset($configuration) && is_array($configuration)) {
            // we don't really need to keep track of the sort order, because PHP will fill in the array in the right order !?
            if (isset($configuration['_sortorder_'])) {
               unset($configuration['_sortorder_']);
            }
            $newconfig = array();
            $idx = 1;
            foreach ($configuration as $profile) {
                // skip entries with empty site or profile
                if (empty($profile[0]) || empty($profile[2])) continue;

                // extract the {variables} from the profile and set variables array
                preg_match_all('/\{([^}]+)\}/', $profile[2], $matches);
                $profile[3] = $matches[1];

                // set the profile to active or inactive
                if (!empty($profile[5])) {
                    $profile[5] = 1;
                } else {
                    $profile[5] = 0;
                }

                // set the sort order (not really needed here)
                $profile[6] = $idx;
                $idx++;

                $newconfig[] = $profile;
            }
            $this->configuration = serialize($newconfig);
        }
        return true;
    }

    public function defaultProfiles()
    {
        // Format:
        // array([0] => site,       // name of the site
        //       [1] => url,        // link to the site
        //       [2] => profile,    // profile template with variable name(s)
        //       [3] => variables,  // array of variable name(s)
        //       [4] => icon,       // icon in *roles* xarimages
        //       [5] => active,     // active/disabled
        //       [6] => order)      // display order (not really needed here)
        $profiles = array(
            array('E-Mail',
                  '',
                  'mailto:{email}',
                  array('email'),
                  'contact/mail.png',
                  1,
                  1),
            array('Web Site',
                  '',
                  '{website}',
                  array('website'),
                  'contact/web.png',
                  1,
                  2),
/*
            array('Affero',
                  'http://www.affero.com/',
                  'http://svcs.affero.net/user-history.php?ll=lq_members&u={username}',
                  array('username'),
                  'contact/affero.png',
                  1,
                  3),
*/
            array('AIM',
                  'http://www.aim.com/',
                  'aim:goim?screenname={username}',
                  array('username'),
                  'contact/aim.png',
                  1,
                  4),
            array('ICQ',
                  'http://www.icq.com/',
                  'http://www.icq.com/whitepages/cmd.php?uin={username}&action=message',
                  array('username'),
                  'contact/icq.png',
                  1,
                  5),
            array('Jabber',
                  'http://www.jabber.org/',
                  'xmpp:{username}?message',
                  array('username'),
                  'contact/jabber.png',
                  1,
                  6),
            array('MSN Messenger',
                  'http://www.msn.com/',
                  'msnim:chat?contact={username}',
                  array('username'),
                  'contact/msnm.png',
                  1,
                  7),
            array('Skype',
                  'http://www.skype.com/',
                  'skype:{username}?call',
                  array('username'),
                  'contact/skype.png',
                  1,
                  8),
            array('Yahoo! Messenger',
                  'http://messenger.yahoo.com/',
                  'ymsgr:sendIM?{username}',
                  array('username'),
                  'contact/yahoo.png',
                  1,
                  9),
            array('StumbleUpon',
                  'http://www.stumbleupon.com/',
                  'http://{username}.stumbleupon.com/',
                  array('username'),
                  'social/stumbleupon.png',
                  1,
                  10),
            array('Twitter',
                  'http://www.twitter.com/',
                  'http://www.twitter.com/{username}',
                  array('username'),
                  'social/twitter.png',
                  1,
                  11),
            array('Plurk',
                  'http://www.plurk.com/',
                  'http://www.plurk.com/user/{username}/',
                  array('username'),
                  'social/plurk.png',
                  1,
                  12),
            array('Ma.gnolia',
                  'http://ma.gnolia.com/',
                  'http://ma.gnolia.com/people/{username}/',
                  array('username'),
                  'social/magnolia.png',
                  1,
                  13),
            array('Flickr',
                  'http://www.flickr.com/',
                  'http://www.flickr.com/photos/{username}/',
                  array('username'),
                  'social/flickr.png',
                  1,
                  14),
            array('Delicious',
                  'http://delicious.com/',
                  'http://delicious.com/{username}/',
                  array('username'),
                  'social/delicious.png',
                  1,
                  15),
            array('Facebook',
                  'http://www.facebook.com/',
                  'http://www.facebook.com/people/{name}/{userid}',
                  array('name','userid'),
                  'social/facebook.png',
                  1,
                  16),
            array('Friendfeed',
                  'http://friendfeed.com',
                  'http://friendfeed.com/{username}',
                  array('username'),
                  'social/friendfeed.png',
                  1,
                  17),
            array('Lifestream',
                  'http://lifestream.fm/',
                  'http://lifestream.fm/{username}',
                  array('username'),
                  'social/lifestream.png',
                  1,
                  18),
            array('LinkedIn',
                  'http://www.linkedin.com/',
                  'http://www.linkedin.com/in/{username}',
                  array('username'),
                  'social/linkedin.png',
                  1,
                  19),
            array('MySpace',
                  'http://www.myspace.com/',
                  'http://www.myspace.com/{username}',
                  array('username'),
                  'social/myspace.png',
                  1,
                  20),
            array('Sphinn',
                  'http://www.sphinn.com/',
                  'http://sphinn.com/user/view/profile/{username}/',
                  array('username'),
                  'social/sphinn.png',
                  1,
                  21),
            array('MyOpenId',
                  'http://www.myopenid.com/',
                  'http://{username}.myopenid.com/',
                  array('username'),
                  'social/myopenid.png',
                  1,
                  22),
            array('YouTube',
                  'http://www.youtube.com/',
                  'http://www.youtube.com/{username}',
                  array('username'),
                  'social/youtube.png',
                  1,
                  23),
            array('ThisNext',
                  'http://www.thisnext.com/',
                  'http://www.thisnext.com/by/{username}/',
                  array('username'),
                  'social/thisnext.png',
                  1,
                  24),
            array('Digg',
                  'http://digg.com/',
                  'http://digg.com/users/{username}/',
                  array('username'),
                  'social/digg.png',
                  1,
                  25),
            array('ThinkAtheist',
                  'http://www.thinkatheist.com/',
                  'http://www.thinkatheist.com/profile/{username}/',
                  array('username'),
                  'social/thinkatheist.png',
                  1,
                  26),
            array('netlog',
                  'http://www.netlog.com/',
                  'http://www.netlog.com/{username}/',
                  array('username'),
                  'social/netlog.png',
                  1,
                  27),
            array('Barcodepedia',
                  'http://www.barcodepedia.com/',
                  'http://www.barcodepedia.com/users/{username}/',
                  array('username'),
                  'social/barcodepedia.png',
                  1,
                  28),
            array('Steam',
                  'http://steamcommunity.com/',
                  'http://steamcommunity.com/id/{username}/',
                  array('username'),
                  'social/steam.png',
                  1,
                  29),
            array('Last.fm',
                  'http://www.last.fm',
                  'http://www.last.fm/user/{username}',
                  array('username'),
                  'social/lastfm.png',
                  1,
                  30),
            array('Reddit',
                  'http://www.reddit.com/',
                  'http://www.reddit.com/user/{username}/',
                  array('username'),
                  'social/reddit.png',
                  1,
                  31),
            array('Furl',
                  'http://www.furl.net/',
                  'http://www.furl.net/member/{username}/',
                  array('username'),
                  'social/furl.png',
                  1,
                  32),
            array('Mister Wong',
                  'http://www.mister-wong.com/',
                  'http://www.mister-wong.com/user/{username}/',
                  array('username'),
                  'social/misterwong.png',
                  1,
                  33),
            array('Twine',
                  'http://www.twine.com/',
                  'http://www.twine.com/user/{username}/',
                  array('username'),
                  'social/twine.png',
                  1,
                  34),
            array('Wis.dm',
                  'http://wis.dm/',
                  'http://wis.dm/users/{userid}-{username}/',
                  array('userid','username'),
                  'social/wisdm.png',
                  1,
                  35),
            array('SWiK',
                  'http://swik.net/',
                  'http://swik.net/User:{username}/',
                  array('username'),
                  'social/swik.png',
                  1,
                  36),
            array('Squidoo',
                  'http://www.squidoo.com/',
                  'http://www.squidoo.com/lensmasters/{username}',
                  array('username'),
                  'social/squidoo.png',
                  1,
                  37),
            array('Simpy',
                  'http://www.simpy.com/',
                  'http://www.simpy.com/user/{username}',
                  array('username'),
                  'social/simpy.png',
                  1,
                  38),
            array('SEOmoz',
                  'http://www.seomoz.org/',
                  'http://www.seomoz.org/users/view/{username}/',
                  array('username'),
                  'social/seomoz.png',
                  1,
                  39),
            array('Newsvine',
                  'http://www.newsvine.com/',
                  'http://{username}.newsvine.com/',
                  array('username'),
                  'social/newsvine.png',
                  1,
                  40),
            array('Netvouz',
                  'http://www.netvouz.com/',
                  'http://www.netvouz.com/{username}/',
                  array('username'),
                  'social/netvouz.png',
                  1,
                  41),
            array('Faves',
                  'http://faves.com/',
                  'http://faves.com/users/{username}',
                  array('username'),
                  'social/faves.png',
                  1,
                  42),
            array('Diigo',
                  'http://www.diigo.com/',
                  'http://www.diigo.com/profile/{username}/',
                  array('username'),
                  'social/diigo.png',
                  1,
                  43),
            array('de.lirio.us',
                  'http://de.lirio.us/',
                  'http://de.lirio.us/profile.php/{username}/',
                  array('username'),
                  'social/delirious.png',
                  1,
                  44),
            array('BlinkList',
                  'http://www.blinklist.com/',
                  'http://www.blinklist.com/{username}/',
                  array('username'),
                  'social/blinklist.png',
                  1,
                  45),
            array('BibSonomy',
                  'http://www.bibsonomy.org/',
                  'http://www.bibsonomy.org/user/{username}',
                  array('username'),
                  'social/bigsonomy.png',
                  1,
                  46),
            array('MyBlogLog',
                  'http://www.mybloglog.com/',
                  'http://www.mybloglog.com/buzz/members/{username}/',
                  array('username'),
                  'social/mybloglog.png',
                  1,
                  47),
            array('bebo',
                  'http://www.bebo.com/',
                  'http://www.bebo.com/{username}/',
                  array('username'),
                  'social/bebo.png',
                  1,
                  48),
            array('Friendster',
                  'http://www.friendster.com/',
                  'http://profiles.friendster.com/{username}/',
                  array('username'),
                  'social/friendster.png',
                  1,
                  49),
            array('DOPPLR',
                  'http://www.dopplr.com/',
                  'http://www.dopplr.com/traveller/{username}/',
                  array('username'),
                  'social/dopplr.png',
                  1,
                  50),
            array('Jaiku',
                  'http://www.jaiku.com/',
                  'http://{username}.jaiku.com/',
                  array('username'),
                  'social/jaiku.png',
                  1,
                  51),
            array('Jumpcut',
                  'http://www.jumpcut.com/',
                  'http://www.jumpcut.com/{username}',
                  array('username'),
                  'social/jumpcut.png',
                  1,
                  52),
            array('LiveJournal',
                  'http://www.livejournal.com/',
                  'http://{username}.livejournal.com/',
                  array('username'),
                  'social/ilivejournal.png',
                  1,
                  53),
            array('Multiply',
                  'http://www.multiply.com/',
                  'http://{username}.multiply.com/',
                  array('username'),
                  'social/multiply.png',
                  1,
                  54),
            array('Netflix',
                  'http://www.netflix.com/',
                  'http://rss.netflix.com/QueueRSS?id={username}',
                  array('username'),
                  'social/netflix.png',
                  1,
                  55),
            array('Shelfari',
                  'http://www.shelfari.com/',
                  'http://www.shelfari.com/{username}',
                  array('username'),
                  'social/shelfari.png',
                  1,
                  56),
            array('SlideShare',
                  'http://www.slideshare.net/',
                  'http://www.slideshare.net/{username}/',
                  array('username'),
                  'social/slideshare.png',
                  1,
                  57),
            array('Technorati',
                  'http://www.technorati.com/',
                  'http://www.technorati.com/people/technorati/{username}/',
                  array('username'),
                  'social/technorati.png',
                  1,
                  58),
            array('The DJ List',
                  'http://www.thedjlist.com/',
                  'http://www.thedjlist.com/djs/{username}/',
                  array('username'),
                  'social/thedjlist.png',
                  1,
                  59),
            array('Upcoming',
                  'http://upcoming.yahoo.com/',
                  'http://upcoming.yahoo.com/user/{username}/',
                  array('username'),
                  'social/upcoming.png',
                  1,
                  60),
            array('Wakoopa',
                  'http://wakoopa.com/',
                  'http://wakoopa.com/{username}/',
                  array('username'),
                  'social/wakoopa.png',
                  1,
                  61),
            array('Yelp',
                  'http://www.yelp.com/',
                  'http://www.yelp.com/user_details?userid={username}',
                  array('username'),
                  'social/yelp.png',
                  1,
                  62),
            array('Zorpia',
                  'http://www.zorpia.com/',
                  'http://www.zorpia.com/{username}/',
                  array('username'),
                  'social/zorpia.png',
                  1,
                  63),
            array('BlogMarks',
                  'http://www.blogmarks.net/',
                  'http://www.blogmarks.net/user/{username}/',
                  array('username'),
                  'social/blogmarks.png',
                  1,
                  64),
            array('BlogMemes',
                  'http://www.blogmemes.com/',
                  'http://www.blogmemes.com/user/{username}',
                  array('username'),
                  'social/blogmemes.png',
                  1,
                  65),
            array('Blogosphere News',
                  'http://www.blogospherenews.com/',
                  'http://www.blogospherenews.com/user.php?login={username}',
                  array('username'),
                  'social/blogospherenews.gif',
                  1,
                  66),
            array('Blogsvine',
                  'http://www.blogsvine.com/',
                  'http://www.blogsvine.com/user/view/profile/{username}/',
                  array('username'),
                  'social/blogsvine.png',
                  1,
                  67),
            array('brightkite',
                  'http://brightkite.com/',
                  'http://brightkite.com/people/{username}/',
                  array('username'),
                  'social/brightkite.png',
                  1,
                  68),
            array('BUMPzee',
                  'http://www.bumpzee.com/',
                  'http://www.bumpzee.com/users/view/{username}/',
                  array('username'),
                  'social/bumpzee.png',
                  1,
                  69),
            array('Dailymotion',
                  'http://www.dailymotion.com/',
                  'http://www.dailymotion.com/{username}/',
                  array('username'),
                  'social/dailymotion.png',
                  1,
                  70),
            array('Clipmarks',
                  'http://www.clipmarks.com/clipper/Torley/',
                  'http://www.clipmarks.com/clipper/{username}/',
                  array('username'),
                  'social/clipmarks.png',
                  1,
                  71),
            array('Connotea',
                  'http://www.connotea.org/',
                  'http://www.connotea.org/wiki/User:{username}',
                  array('username'),
                  'social/connotea.png',
                  1,
                  72),
            array('Design Float',
                  'http://www.designfloat.com/',
                  'http://www.designfloat.com/user/view/profile/{username}/',
                  array('username'),
                  'social/designfloat.gif',
                  1,
                  73),
            array('DotNetKicks',
                  'http://www.dotnetkicks.com/',
                  'http://www.dotnetkicks.com/users/{username}/',
                  array('username'),
                  'social/dotnetkicks.png',
                  1,
                  74),
            array('DZone',
                  'http://www.dzone.com/',
                  'http://www.dzone.com/links/users/profile/{username}.html',
                  array('username'),
                  'social/dzone.png',
                  1,
                  75),
            array('eKudos',
                  'http://www.ekudos.nl/',
                  'http://www.ekudos.nl/gebruiker/{username}/',
                  array('username'),
                  'social/ekudos.gif',
                  1,
                  76),
            array('FARK',
                  'http://www.fark.com/',
                  'http://cgi.fark.com/cgi/fark/users.pl?login={username}',
                  array('username'),
                  'social/fark.png',
                  1,
                  77),
            array('Feed Me Links',
                  'http://www.feedmelinks.com/',
                  'http://www.feedmelinks.com/u/{username}/',
                  array('username'),
                  'social/feedmelinks.png',
                  1,
                  78),
            array('Fleck',
                  'http://www.fleck.com/',
                  'http://www.fleck.com/profile/{username}/',
                  array('username'),
                  'social/fleck.gif',
                  1,
                  79),
/*
            array('FoodFeed',
                  'http://www.foodfeed.us/',
                  'http://{username}.foodfeed.us/',
                  array('username'),
                  'social/foodfeed.png',
                  1,
                  80),
*/
            array('Global Grind',
                  'http://globalgrind.com/',
                  'http://globalgrind.com/user/{username}/',
                  array('username'),
                  'social/globalgrind.gif',
                  1,
                  81),
            array('Health Ranker',
                  'http://www.healthranker.com/',
                  'http://www.healthranker.com/user/view/profile/{username}/',
                  array('username'),
                  'social/healthranker.gif',
                  1,
                  82),
            array('HEMiDEMi',
                  'http://www.hemidemi.com/',
                  'http://www.hemidemi.com/user/{username}/',
                  array('username'),
                  'social/hemidemi.png',
                  1,
                  83),
            array('identi.ca',
                  'http://identi.ca/',
                  'http://identi.ca/{username}',
                  array('username'),
                  'social/identica.png',
                  1,
                  84),
            array('iLike',
                  'http://www.ilike.com/',
                  'http://www.ilike.com/user/{username}/',
                  array('username'),
                  'social/ilike.png',
                  1,
                  85),
            array('iliketotallyloveit',
                  'http://www.iliketotallyloveit.com/',
                  'http://www.iliketotallyloveit.com/user/{username}/',
                  array('username'),
                  'social/iliketotallyloveit.png',
                  1,
                  86),
            array('IndianPad',
                  'http://www.indianpad.com/',
                  'http://www.indianpad.com/user/{username}/',
                  array('username'),
                  'social/indianpad.png',
                  1,
                  87),
            array('joinR',
                  'http://www.joinr.de/',
                  'http://{username}.joinr.de/',
                  array('username'),
                  'social/joinr.png',
                  1,
                  88),
            array('Joost',
                  'http://www.joost.com/',
                  'http://www.joost.com/users/{username}/',
                  array('username'),
                  'social/joost.png',
                  1,
                  89),
            array('kirtsy',
                  'http://www.kirtsy.com/',
                  'http://www.kirtsy.com/user.php?login={username}',
                  array('username'),
                  'social/kirtsy.gif',
                  1,
                  90),
            array('KWICK',
                  'http://www.kwick.de/',
                  'http://www.kwick.de/{username}/',
                  array('username'),
                  'social/kwick.png',
                  1,
                  91),
            array('LinkARENA',
                  'http://linkarena.com/',
                  'http://{username}.linkarena.com/',
                  array('username'),
                  'social/linkarena.gif',
                  1,
                  92),
            array('BLOGTER',
                  'http://www.blogter.hu/',
                  'http://www.blogter.hu/index.php?action=view_profile&user_id={username}',
                  array('username'),
                  'social/linkter.png',
                  1,
                  93),
            array('Google Reader',
                  'http://www.google.com/',
                  'http://www.google.com/reader/shared/{username}',
                  array('username'),
                  'social/googlereader.png',
                  1,
                  94),
            array('men&eacute;ame',
                  'http://meneame.net/',
                  'http://meneame.net/user/{username}/',
                  array('username'),
                  'social/meneame.png',
                  1,
                  95),
            array('ebay',
                  'http://www.ebay.com/',
                  'http://myworld.ebay.com/{username}/',
                  array('username'),
                  'social/ebay.png',
                  1,
                  96),
            array('Mixx',
                  'http://www.mixx.com/',
                  'http://www.mixx.com/users/{username}',
                  array('username'),
                  'social/mixx.png',
                  1,
                  97),
            array('muti',
                  'http://www.muti.co.za/',
                  'http://www.muti.co.za/user?name={username}',
                  array('username'),
                  'social/muti.png',
                  1,
                  98),
            array('myvideo',
                  'http://www.myvideo.co.za/',
                  'http://www.myvideo.co.za/channels/{username}',
                  array('username'),
                  'social/myvideo.png',
                  1,
                  99),
            array('N4G',
                  'http://www.n4g.com',
                  'http://www.n4g.com/up/{username}.aspx',
                  array('username'),
                  'social/n4g.gif',
                  1,
                  100),
            array('Netvibes',
                  'http://www.netvibes.com/',
                  'http://www.netvibes.com/{username}',
                  array('username'),
                  'social/netvibes.png',
                  1,
                  101),
            array('Odeo',
                  'http://www.odeo.com/',
                  'http://www.odeo.com/users/{username}/',
                  array('username'),
                  'social/odeo.png',
                  1,
                  102),
            array('Photocase',
                  'http://www.photocase.com/',
                  'http://www.photocase.com/en/user.asp?u={username}',
                  array('username'),
                  'social/photocase.png',
                  1,
                  103),
            array('Picasaweb',
                  'http://picasaweb.google.com/',
                  'http://picasaweb.google.com/{username}/',
                  array('username'),
                  'social/picasa.png',
                  1,
                  104),
            array('Plazes',
                  'http://plazes.com/',
                  'http://plazes.com/users/{username}/',
                  array('username'),
                  'social/plazes.png',
                  1,
                  105),
            array('PlugIM',
                  'http://www.plugim.com/',
                  'http://www.plugim.com/user/{username}/',
                  array('username'),
                  'social/plugim.png',
                  1,
                  106),
            array('Polyvore',
                  'http://www.polyvore.com/',
                  'http://www.polyvore.com/cgi/profile?id={username}',
                  array('username'),
                  'social/polyvore.png',
                  1,
                  107),
            array('ppnow',
                  'http://www.ppnow.com/',
                  'http://www.ppnow.com/bookmark/{username}/',
                  array('username'),
                  'social/ppnow.png',
                  1,
                  108),
            array('Propeller',
                  'http://www.propeller.com/',
                  'http://www.propeller.com/member/{username}/',
                  array('username'),
                  'social/propeller.png',
                  1,
                  109),
            array('QIK',
                  'http://qik.com/',
                  'http://qik.com/{username}/',
                  array('username'),
                  'social/qik.png',
                  1,
                  110),
            array('Qype',
                  'http://www.qype.co.uk/',
                  'http://www.qype.co.uk/people/{username}/',
                  array('username'),
                  'social/qype.png',
                  1,
                  111),
            array('Ratimarks',
                  'http://www.ratimarks.org/',
                  'http://www.ratimarks.org/bookmarks.php/{username}/',
                  array('username'),
                  'social/ratimarks.png',
                  1,
                  112),
            array('Scoopeo',
                  'http://www.scoopeo.com/',
                  'http://www.scoopeo.com/membre/{username}/',
                  array('username'),
                  'social/scoopeo.png',
                  1,
                  113),
            array('Segnalo',
                  'http://segnalo.alice.it/',
                  'http://segnalo.alice.it/profile/{username}',
                  array('username'),
                  'social/segnalo.gif',
                  1,
                  114),
            array('Sevenload',
                  'http://www.sevenload.com/',
                  'http://www.sevenload.com/users/{username}',
                  array('username'),
                  'social/sevenload.png',
                  1,
                  115),
            array('Sixgroups',
                  'http://sixgroups.com/',
                  'http://sixgroups.com/profile/{username}/',
                  array('username'),
                  'social/sixgroups.png',
                  1,
                  116),
            array('Slashdot',
                  'http://slashdot.org/',
                  'http://slashdot.org/~{username}',
                  array('username'),
                  'social/slashdot.png',
                  1,
                  117),
            array('SmugMug',
                  'http://www.smugmug.com',
                  'http://{username}.smugmug.com/',
                  array('username'),
                  'social/smugmug.png',
                  1,
                  118),
            array('Folkd',
                  'http://www.folkd.com/',
                  'http://www.folkd.com/user/{username}/',
                  array('username'),
                  'social/folkd.png',
                  1,
                  119),
            array('tipjoy',
                  'http://tipjoy.com/',
                  'http://tipjoy.com/u/{username}/',
                  array('username'),
                  'social/tipjoy.png',
                  1,
                  120),
            array('Tumblr',
                  'http://www.tumblr.com/',
                  'http://{username}.tumblr.com/',
                  array('username'),
                  'social/tumblr.png',
                  1,
                  121),
            array('Twitxr',
                  'http://www.twitxr.com/',
                  'http://www.twitxr.com/{username}/',
                  array('username'),
                  'social/twitxr.png',
                  1,
                  122),
            array('uboot',
                  'http://www.uboot.com/',
                  'http://{username}.uboot.com/',
                  array('username'),
                  'social/uboot.png',
                  1,
                  123),
            array('Upnews',
                  'http://www.upnews.it/',
                  'http://www.upnews.it/user/view/profile/{username}/',
                  array('username'),
                  'social/upnews.gif',
                  1,
                  124),
            array('Venteria',
                  'http://venteria.com/',
                  'http://venteria.com/users/{username}/',
                  array('username'),
                  'social/venteria.png',
                  1,
                  125),
            array('Vimeo',
                  'http://www.vimeo.com/',
                  'http://www.vimeo.com/{username}/',
                  array('username'),
                  'social/vimeo.png',
                  1,
                  126),
            array('vi.sualize.us',
                  'http://vi.sualize.us/',
                  'http://vi.sualize.us/{username}/',
                  array('username'),
                  'social/visualizeus.png',
                  1,
                  127),
            array('Webride',
                  'http://webride.org/',
                  'http://webride.org/users/{username}/',
                  array('username'),
                  'social/webride.png',
                  1,
                  128),
            array('weheartit',
                  'http://www.weheartit.com/',
                  'http://www.weheartit.com/user/{username}/',
                  array('username'),
                  'social/weheartit.png',
                  1,
                  129),
            array('Wikipedia',
                  'http://www.wikipedia.org/',
                  'http://www.wikipedia.org/wiki/User:{username}',
                  array('username'),
                  'social/wikipedia.png',
                  1,
                  130),
            array('Wists',
                  'http://www.wists.com/',
                  'http://www.wists.com/{username}/',
                  array('username'),
                  'social/wists.png',
                  1,
                  131),
            array('Woobby',
                  'http://de.woobby.com/',
                  'http://de.woobby.com/user/profile?username={username}',
                  array('username'),
                  'social/woobby.png',
                  1,
                  132),
            array('xerpi',
                  'http://www.xerpi.com/',
                  'http://www.xerpi.com/profile/{username}/',
                  array('username'),
                  'social/xerpi.gif',
                  1,
                  133),
            array('Xing',
                  'http://www.xing.com/',
                  'http://www.xing.com/profile/{username}',
                  array('username'),
                  'social/xing.png',
                  1,
                  134),
            array('YiGG',
                  'http://www.yigg.de/',
                  'http://www.yigg.de/profil/{username}',
                  array('username'),
                  'social/yigg.png',
                  1,
                  135),
            array('Zooomr',
                  'http://www.zooomr.com/',
                  'http://www.zooomr.com/photos/{username}/',
                  array('username'),
                  'social/zooomr.png',
                  1,
                  136),
            array('Clipmarks',
                  'http://www.clipmarks.com/',
                  'http://www.clipmarks.com/clipper/{username}/',
                  array('username'),
                  'social/zooomr.png',
                  1,
                  137),
            array('Blogger',
                  'http://www.blogger.com/',
                  'http://www.blogger.com/profile/{username}',
                  array('username'),
                  'social/blogger.png',
                  1,
                  138),
            array('Goodreads',
                  'http://www.goodreads.com/',
                  'http://www.goodreads.com/profile/{userid}/',
                  array('userid'),
                  'social/goodreads.png',
                  1,
                  139),
            array('Disqus',
                  'http://www.disqus.com/',
                  'http://www.disqus.com/people/{username}/',
                  array('username'),
                  'social/disqus.png',
                  1,
                  140),
            array('Hi5',
                  'http://www.hi5.com/',
                  'http://www.hi5.com/friend/p{userid}--{username}--html',
                  array('userid','username'),
                  'social/hi5.png',
                  1,
                  141),
            array('ohloh',
                  'http://www.ohloh.net/',
                  'http://www.ohloh.net/accounts/{username}',
                  array('username'),
                  'social/ohloh.png',
                  1,
                  142),
            array('Writing.com',
                  'http://www.writing.com/',
                  'http://www.writing.com/main/view_item/user_id/{username}',
                  array('username'),
                  'social/writing.png',
                  1,
                  143),
            array('deviantART',
                  'http://www.deviantart.com/',
                  'http://{username}.deviantart.com/',
                  array('username'),
                  'social/deviantart.png',
                  1,
                  144),
            array('Blip.fm',
                  'http://blip.fm/',
                  'http://blip.fm/{username}/',
                  array('username'),
                  'social/blipfm.png',
                  1,
                  145),
            array('Blip.tv',
                  'http://blip.tv/',
                  'http://{username}.blip.tv/',
                  array('username'),
                  'social/bliptv.png',
                  1,
                  146),
            array('Windows Live',
                  'http://profile.live.com/',
                  'http://{userid}.profile.live.com/ ',
                  array('userid'),
                  'social/livecom.png',
                  1,
                  147),
            array('WAYN',
                  'http://www.wayn.com/',
                  'http://www.wayn.com/waynprofile.html?member_key={userid} ',
                  array('userid'),
                  'social/wayncom.png',
                  1,
                  148),
            array('Facebook Group',
                  'http://www.facebook.com/',
                  'http://www.facebook.com/group.php?gid={groupid}',
                  array('groupid'),
                  'social/facebook.png',
                  1,
                  149),
            array('Facebook Page',
                  'http://www.facebook.com/',
                  'http://www.facebook.com/pages/{pagename}/{pageid}',
                  array('pagename','pageid'),
                  'social/facebook.png',
                  1,
                  150),
            array('LinkedIn Company',
                  'http://www.linkedin.com/',
                  'http://www.linkedin.com/companies/{lkdncompanynameid}/{lkdncompanyname}',
                  array('lkdncompanynameid','lkdncompanyname'),
                  'social/linkedin.png',
                  1,
                  151),
            array('imeem',
                  'http://www.imeem.com/',
                  'http://www.imeem.com/{username}',
                  array('username'),
                  'social/imeem.png',
                  1,
                  152),
            array('12seconds',
                  'http://12seconds/',
                  'http://12seconds.tv/channel/{username}',
                  array('username'),
                  'social/12secondstv.png',
                  1,
                  153),
            array('Behance',
                  'http://www.behance.net/',
                  'http://www.behance.net/{username}',
                  array('username'),
                  'social/behancenet.png',
                  1,
                  154),
            array('LibraryThing',
                  'http://www.librarything.com/',
                  'http://www.librarything.com/profile/{username}',
                  array('username'),
                  'social/librarythingcom.png',
                  1,
                  155),
            array('LibraryThing Author',
                  'http://www.librarything.com/',
                  'http://www.librarything.com/author/{username}',
                  array('username'),
                  'social/librarythingcom.png',
                  1,
                  156),
            array('Facebook Profile',
                  'http://www.facebook.com/',
                  'http://www.facebook.com/{username}',
                  array('username'),
                  'social/facebook.png',
                  1,
                  157),
            array('Hyves',
                  'http://www.hyves.co.uk/',
                  'http://{username}.hyves.co.uk/',
                  array('username'),
                  'social/hyvescom.png',
                  1,
                  158),
            array('ClaimID',
                  'http://claimid.com/',
                  'http://claimid.com/{username}',
                  array('username'),
                  'social/claimidcom.png',
                  1,
                  159),
        );

        return $profiles;
    }
}
?>